@@ -1,6 +1,7 @@
 /*
 This is part of the audio CD player library
-Copyright (C)1998-99 Tony Arcieri
+Copyright (C)1998-99 Tony Arcieri <bascule@inferno.tusculum.edu>
+Parts Copyright (C)1999 Quinton Dolan <q@OntheNet.com.au>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -24,6 +25,7 @@ Boston, MA  02111-1307, USA.
 #include <stdarg.h>
 #include <string.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -41,6 +43,18 @@ Boston, MA  02111-1307, USA.
 #include <sys/mntent.h>
 #endif
 
+#ifdef SOLARIS_GETMNTENT
+#include <sys/mnttab.h>
+#endif
+
+#ifdef HAVE_SYS_UCRED_H
+#include <sys/ucred.h>
+#endif
+
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
 /* For Linux */
 #ifdef HAVE_LINUX_CDROM_H
 #include <linux/cdrom.h>
@@ -61,9 +75,7 @@ Boston, MA  02111-1307, USA.
 #include <io/cam/cdrom.h>
 #endif
 
-#ifdef CDIOCREADSUBCHANNEL
-#define CDIOREADSUBCHANNEL CDIOCREADSUBCHANNEL
-#endif
+#include "compat.h"
 
 
 #ifndef IRIX_CDAUDIO
@@ -85,6 +97,12 @@ cd_version(buffer, len)
    snprintf(buffer, len, "%s %s", PACKAGE, VERSION);
 }
 
+/* Return the version of libcdaudio */
+long cdaudio_getversion(void)
+{
+   return LIBCDAUDIO_VERSION;
+}
+
 /* Initialize the CD-ROM for playing audio CDs */
 int
 #if __STDC__
@@ -96,25 +114,51 @@ cd_init_device(device_name)
 {
    int cd_desc;
 
-#ifdef HAVE_GETMNTENT
+#ifdef SOLARIS_GETMNTENT
+   FILE *mounts;
+   struct mnttab mnt;
+#elif defined(HAVE_GETMNTENT)
    FILE *mounts;
    struct mntent *mnt;
-   char devname[256];
+#endif
+#ifdef HAVE_GETMNTINFO
+   int mounts;
+   struct statfs *mnt;
+#endif
+   char devname[255];
    struct stat st;
+   int len = 0;
 
    if(lstat(device_name, &st) < 0)
      return -1;
    
-   if(S_ISLNK(st.st_mode))
-     readlink(device_name, devname, 256);
-   else
-     strncpy(devname, device_name, 256);
+   if(S_ISLNK(st.st_mode)) {
+      /* Some readlink implementations don't put a \0 on the end */
+      len = readlink(device_name, devname, 255);
+      devname[len] = '\0';
+   } else {
+      strncpy(devname, device_name, 255);
+      len = strlen(devname);
+   }
+#ifdef SOLARIS_GETMNTENT
+   if((mounts = fopen(MNTTAB, "r")) == NULL)
+      return -1;
 
+   while(getmntent(mounts, &mnt) == 0) {
+      if(strncmp(mnt.mnt_fstype, devname, len) == 0) {
+         errno = EBUSY;
+         return -1;
+      }
+   }
+
+   if(fclose(mounts) != 0)
+      return -1;
+#elif defined(HAVE_GETMNTENT)
    if((mounts = setmntent(MOUNTED, "r")) == NULL)
       return -1;
       
    while((mnt = getmntent(mounts)) != NULL) {
-      if(strcmp(mnt->mnt_fsname, devname) == 0) {
+      if(strncmp(mnt->mnt_fsname, devname, len) == 0) {
 	 endmntent(mounts);
 	 errno = EBUSY;
 	 return -1;
@@ -122,6 +166,17 @@ cd_init_device(device_name)
    }
    endmntent(mounts);
 #endif
+#ifdef HAVE_GETMNTINFO
+   for ( (mounts = getmntinfo(&mnt, 0)); mounts > 0;)
+   {
+      mounts--;
+      if (strncmp(mnt[mounts].f_mntfromname, devname, len) == 0)
+      {
+         errno = EBUSY;
+         return -1;
+      }
+   }
+#endif   
 
 #ifdef NON_BLOCKING
    if((cd_desc = open(device_name, O_RDONLY | O_NONBLOCK)) < 0)
@@ -159,26 +214,13 @@ cd_stat(cd_desc, disc)
 {
    /* Since every platform does this a little bit differently this gets pretty
       complicated... */
-#ifdef CDIOREADTOCHEADER
-   struct ioc_toc_header cdth;
-#endif
-#ifdef CDIOREADTOCENTRYS
-   struct cd_toc_entry toc_buffer[MAX_TRACKS];
-   struct ioc_read_toc_entry cdte;
-#endif
-#ifdef CDROMREADTOCHDR
-   struct cdrom_tochdr cdth;
-#endif
-#ifdef CDROMREADTOCENTRY
-   struct cdrom_tocentry cdte;
-#endif
-#ifdef CDROM_TOC_HEADER
-   struct cd_toc_header th;
-#endif
-#ifdef CDROM_TOC_ENTRYS
-   struct cd_toc_entry te[MAX_TRACKS];
-   struct cd_toc toc;
+
+   struct CDAUDIO_TOCHEADER cdth;
+   struct CDAUDIO_TOCENTRY cdte;
+#ifdef CDAUDIO_TOCENTRY_DATA
+   struct CDAUDIO_TOCENTRY_DATA cdte_buffer[MAX_TRACKS];
 #endif
+
    struct disc_status status;
    int readtracks, pos;
    
@@ -191,96 +233,51 @@ cd_stat(cd_desc, disc)
    }
    
    /* Read the Table Of Contents header */
-#ifdef CDIOREADTOCHEADER
-   if(ioctl(cd_desc, CDIOREADTOCHEADER, &cdth) < 0)
+   if(ioctl(cd_desc, CDAUDIO_READTOCHEADER, &cdth) < 0)
      return -1;
-   
-   disc->disc_first_track = cdth.starting_track;
-   disc->disc_total_tracks = cdth.ending_track;
-#endif
-#ifdef CDROMREADTOCHDR
-   if(ioctl(cd_desc, CDROMREADTOCHDR, &cdth) < 0)
-     return -1;
-   
-   disc->disc_first_track = cdth.cdth_trk0;
-   disc->disc_total_tracks = cdth.cdth_trk1;
-#endif
-#ifdef CDROM_TOC_HEADER
-   if(ioctl(cd_desc, CDROM_TOC_HEADER, &th) < 0)
-     return -1;
-   
-   disc->disc_first_track = th.th_starting_track;
-   disc->disc_total_tracks = th.th_ending_track;
-#endif
-   
+
+   disc->disc_first_track = cdth.CDTH_STARTING_TRACK;
+   disc->disc_total_tracks = cdth.CDTH_ENDING_TRACK;
+
+#ifdef CDAUDIO_READTOCENTRYS
+
    /* Read the Table Of Contents */
-#ifdef CDIOREADTOCENTRYS
-   cdte.address_format = CD_MSF_FORMAT;
-   cdte.starting_track = 0;
-   cdte.data = toc_buffer;
-   cdte.data_len = sizeof(toc_buffer);
-   
-   if(ioctl(cd_desc, CDIOREADTOCENTRYS, &cdte) < 0)
+   cdte.CDTE_ADDRESS_FORMAT = CDAUDIO_MSF_FORMAT;
+   cdte.CDTE_STARTING_TRACK = 0;
+   cdte.CDTE_DATA = cdte_buffer;
+   cdte.CDTE_DATA_LEN = sizeof(cdte_buffer);
+
+   if(ioctl(cd_desc, CDAUDIO_READTOCENTRYS, &cdte) < 0)
      return -1;
-   
+
    for(readtracks = 0; readtracks <= disc->disc_total_tracks; readtracks++) {
-      disc->disc_track[readtracks].track_pos.minutes = cdte.data[readtracks].addr.msf.minute;
-      disc->disc_track[readtracks].track_pos.seconds = cdte.data[readtracks].addr.msf.second;
-      disc->disc_track[readtracks].track_pos.frames = cdte.data[readtracks].addr.msf.frame;
-      disc->disc_track[readtracks].track_type = CDAUDIO_TRACK_AUDIO;
+      disc->disc_track[readtracks].track_pos.minutes = cdte.CDTE_DATA[readtracks].CDTE_MSF_M;
+      disc->disc_track[readtracks].track_pos.seconds = cdte.CDTE_DATA[readtracks].CDTE_MSF_S;
+      disc->disc_track[readtracks].track_pos.frames = cdte.CDTE_DATA[readtracks].CDTE_MSF_F;
+      disc->disc_track[readtracks].track_type = (cdte.CDTE_DATA[readtracks].CDTE_CONTROL & CDROM_DATA_TRACK) ? CDAUDIO_TRACK_DATA : CDAUDIO_TRACK_AUDIO;
+      disc->disc_track[readtracks].track_lba = cd_msf_to_lba(disc->disc_track[readtracks].track_pos);
    }
-   
-   cdte.address_format = CD_LBA_FORMAT;
-   
-   if(ioctl(cd_desc, CDIOREADTOCENTRYS, &cdte) < 0)
-     return -1;
-   
-   for(readtracks = 0; readtracks < disc->disc_total_tracks; readtracks++)
-     disc->disc_track[readtracks].track_lba = ntohl(cdte.data[readtracks].addr.lba);
-   disc->disc_track[disc->disc_total_tracks].track_lba = ntohl(cdte.data[disc->disc_total_tracks].addr.lba) + 1;
-#endif
-#ifdef CDROMREADTOCENTRY
+
+#else /* CDAUDIO_READTOCENTRYS */
+
    for(readtracks = 0; readtracks <= disc->disc_total_tracks; readtracks++) {
-      cdte.cdte_track = (readtracks == disc->disc_total_tracks) ? CDROM_LEADOUT : readtracks + 1;
-      cdte.cdte_format = CDROM_MSF;
-      if(ioctl(cd_desc, CDROMREADTOCENTRY, &cdte) < 0)
-	return -1;
-	 
-      disc->disc_track[readtracks].track_pos.minutes = cdte.cdte_addr.msf.minute;
-      disc->disc_track[readtracks].track_pos.seconds = cdte.cdte_addr.msf.second;
-      disc->disc_track[readtracks].track_pos.frames = cdte.cdte_addr.msf.frame;
-      disc->disc_track[readtracks].track_type = (cdte.cdte_ctrl & CDROM_DATA_TRACK) ? CDAUDIO_TRACK_DATA : CDAUDIO_TRACK_AUDIO;
-      
-      cdte.cdte_format = CDROM_LBA;
-      if(ioctl(cd_desc, CDROMREADTOCENTRY, &cdte) < 0)
-	return -1;
-      
-      disc->disc_track[readtracks].track_lba = cdte.cdte_addr.lba;
+      cdte.CDTE_STARTING_TRACK = (readtracks == disc->disc_total_tracks) ? CDROM_LEADOUT : readtracks + 1;
+      cdte.CDTE_ADDRESS_FORMAT = CDAUDIO_MSF_FORMAT;
+      if(ioctl(cd_desc, CDAUDIO_READTOCENTRY, &cdte) < 0)
+      return -1;
+
+      disc->disc_track[readtracks].track_pos.minutes = cdte.CDTE_MSF_M;
+      disc->disc_track[readtracks].track_pos.seconds = cdte.CDTE_MSF_S;
+      disc->disc_track[readtracks].track_pos.frames = cdte.CDTE_MSF_F;
+      disc->disc_track[readtracks].track_type = (cdte.CDTE_CONTROL & CDROM_DATA_TRACK) ? CDAUDIO_TRACK_DATA : CDAUDIO_TRACK_AUDIO;
+      disc->disc_track[readtracks].track_lba = cd_msf_to_lba(disc->disc_track[readtracks].track_pos);
    }
-#endif
-#ifdef CDROM_TOC_ENTRYS
-   toc.toc_address_format = CDROM_MSF_FORMAT;
-   toc.toc_starting_track = 0;
-   toc.toc_buffer = te;
-   toc.toc_alloc_length = sizeof(te);
-   
-   if(ioctl(cd_desc, CDROM_ROC_ENTRYS, &toc) < 0)
-     return -1;
-   
-   for(readtracks = 0; readtracks <= disc->disc_total_tracks; readtracks++) {
-      disc->disc_track[readtracks].track_pos.minutes = te.te_absaddr.m_units;
-      disc->disc_track[readtracks].track_pos.seconds = te.te_absaddr.s_units;
-      disc->disc_track[readtracks].track_pos.frames = te.te_absaddr.f_units;
-      disc->disc_track[readtracks].track_type = (te.te_control & CDROM_DATA_TRACK) ? CDAUDIO_TRACK_DATA : CDAUDIO_TRACK_AUDIO;
-      disc->disc_track[readtracks].track_lba = disc->disc_track[readtracks].track_pos.minutes * 4500 + disc->disc_track[readtracks].track_pos.seconds * 75 + disc->disc_track[readtracks].track_pos.frames - 150;
-   }
-#endif   
+#endif /* CDAUDIO_READTOCENTRY */
    for(readtracks = 0; readtracks <= disc->disc_total_tracks; readtracks++) {
       if(readtracks > 0) {
-	 pos = ((disc->disc_track[readtracks].track_pos.minutes * 60 + disc->disc_track[readtracks].track_pos.seconds) * 75 + disc->disc_track[readtracks].track_pos.frames) - ((disc->disc_track[readtracks - 1].track_pos.minutes * 60 + disc->disc_track[readtracks - 1].track_pos.seconds) * 75 + disc->disc_track[readtracks - 1].track_pos.frames);
-	 disc->disc_track[readtracks - 1].track_length.minutes = pos / 4500;
-	 disc->disc_track[readtracks - 1].track_length.seconds = (pos % 4500) / 75;
-	 disc->disc_track[readtracks - 1].track_length.frames = pos % 75;
+         pos = cd_msf_to_frames(disc->disc_track[readtracks].track_pos) -
+         	cd_msf_to_frames(disc->disc_track[readtracks - 1].track_pos);
+	 cd_frames_to_msf(&disc->disc_track[readtracks - 1].track_length, pos);
       }
    }
             
@@ -302,41 +299,21 @@ cd_poll(cd_desc, status)
    struct disc_status *status
 #endif
 {
-#ifdef CDIOREADSUBCHANNEL
-   struct ioc_read_subchannel cdsc;
-   struct cd_sub_channel_info data;
+   struct CDAUDIO_SUBCHANNEL cdsc;
+#ifdef CDAUDIO_SUBCHANNEL_DATA
+   struct CDAUDIO_SUBCHANNEL_DATA cdsc_data;
 #endif
-#ifdef CDROMSUBCHNL
-   struct cdrom_subchnl cdsc;
-#endif
-#ifdef CDROM_READ_SUBCHANNEL
-   struct cd_sub_channel sch;
-   struct cd_subc_channel_data scd;
+
+
+#ifdef CDAUDIO_SUBCHANNEL_DATA
+   memset(&cdsc_data, '\0', sizeof(cdsc_data));
+   cdsc.CDSC_DATA = &cdsc_data;
+   cdsc.CDSC_DATA_LEN = sizeof(cdsc_data);
+   cdsc.CDSC_DATA_FORMAT = CDAUDIO_CDSC_DATA_FORMAT;
 #endif
+   cdsc.CDSC_ADDRESS_FORMAT = CDAUDIO_MSF_FORMAT;
    
-#ifdef CDIOREADSUBCHANNEL
-   memset(&cdsc, '\0', sizeof(cdsc));
-   cdsc.data = &data;
-   cdsc.data_len = sizeof(data);
-   cdsc.data_format = CD_CURRENT_POSITION;
-   cdsc.address_format = CD_MSF_FORMAT;
-   
-   if(ioctl(cd_desc, CDIOREADSUBCHANNEL, (char *)&cdsc) < 0)
-#endif
-#ifdef CDROMSUBCHNL
-   cdsc.cdsc_format = CDROM_MSF;
-   
-   if(ioctl(cd_desc, CDROMSUBCHNL, &cdsc) < 0)
-#endif
-#ifdef CDROM_READ_SUBCHANNEL
-   memset(scd, '\0', sizeof(struct cd_subc_channel_data));
-   sch.sch_address_format = CDROM_MSF_FORMAT;
-   sch.sch_data_format = CDROM_SUBQ_DATA;
-   sch.sch_buffer = &scd;
-   sch.sch_alloc_length = sizeof(struct cd_subc_channel_data);
-   
-   if(ioctl(cd_desc, CDROM_READ_SUBCHANNEL, &sch) < 0)
-#endif
+   if(ioctl(cd_desc, CDAUDIO_READSUBCHANNEL, &cdsc) < 0)
      {
 	status->status_present = 0;
 	
@@ -345,64 +322,37 @@ cd_poll(cd_desc, status)
    
    status->status_present = 1;
    
-#ifdef CDIOREADSUBCHANNEL
-   status->status_disc_time.minutes = data.what.position.absaddr.msf.minute;
-   status->status_disc_time.seconds = data.what.position.absaddr.msf.second;
-   status->status_disc_time.frames = data.what.position.absaddr.msf.frame;
-   
-   switch(data.header.audio_status) {
-    case CD_AS_PLAY_IN_PROGRESS:
+#ifdef CDAUDIO_SUBCHANNEL_DATA
+   status->status_disc_time.minutes = cdsc_data.CDSC_DATA_ABS_MSF_M;
+   status->status_disc_time.seconds = cdsc_data.CDSC_DATA_ABS_MSF_S;
+   status->status_disc_time.frames = cdsc_data.CDSC_DATA_ABS_MSF_F;
+   status->status_track_time.minutes = cdsc_data.CDSC_DATA_REL_MSF_M;
+   status->status_track_time.seconds = cdsc_data.CDSC_DATA_REL_MSF_S;
+   status->status_track_time.frames = cdsc_data.CDSC_DATA_REL_MSF_F;
+   status->status_current_track = cdsc_data.CDSC_DATA_TRK;
+   switch(cdsc_data.CDSC_AUDIO_STATUS) {
+#else
+   status->status_disc_time.minutes = cdsc.CDSC_ABS_MSF_M;
+   status->status_disc_time.seconds = cdsc.CDSC_ABS_MSF_S;
+   status->status_disc_time.frames = cdsc.CDSC_ABS_MSF_F;
+   status->status_track_time.minutes = cdsc.CDSC_REL_MSF_M;
+   status->status_track_time.seconds = cdsc.CDSC_REL_MSF_S;
+   status->status_track_time.frames = cdsc.CDSC_REL_MSF_F;
+   status->status_current_track = cdsc.CDSC_TRK;
+   switch(cdsc.CDSC_AUDIO_STATUS) {
+#endif
+    case CDAUDIO_AS_PLAY_IN_PROGRESS:
       status->status_mode = CDAUDIO_PLAYING;
       break;
-    case CD_AS_PLAY_PAUSED:
+    case CDAUDIO_AS_PLAY_PAUSED:
       status->status_mode = CDAUDIO_PAUSED;
       break;
-    case CD_AS_PLAY_COMPLETED:
+    case CDAUDIO_AS_PLAY_COMPLETED:
       status->status_mode = CDAUDIO_COMPLETED;
       break;
     default:
       status->status_mode = CDAUDIO_NOSTATUS;
    }
-#endif
-#ifdef CDROMSUBCHNL
-   status->status_disc_time.minutes = cdsc.cdsc_absaddr.msf.minute;
-   status->status_disc_time.seconds = cdsc.cdsc_absaddr.msf.second;
-   status->status_disc_time.frames = cdsc.cdsc_absaddr.msf.frame;
-   
-   switch(cdsc.cdsc_audiostatus) {
-    case CDROM_AUDIO_PLAY:
-      status->status_mode = CDAUDIO_PLAYING;
-      break;
-    case CDROM_AUDIO_PAUSED:
-      status->status_mode = CDAUDIO_PAUSED;
-      break;
-    case CDROM_AUDIO_COMPLETED:
-      status->status_mode = CDAUDIO_COMPLETED;
-      break;
-    default:
-      status->status_mode = CDAUDIO_NOSTATUS;
-   }
-#endif
-#ifdef CDROM_READ_SUBCHANNEL
-   status->status_disc_time.minutes = scd.scd_position_data.scp_absaddr.m_units;
-   status->status_disc_time.seconds = scd.scd_position_data.scp_absaddr.s_units;
-   status->status_disc_time.frames = scd.scd_position_data.scp_absaddr.f_units;
-   
-   switch(scd.scd_header.sh_audio_status) {
-    case AS_PLAY_IN_PROGRESS:
-      status->status_mode = CDAUDIO_PLAYING;
-      break;
-    case AS_PLAY_PAUSED:
-      status->status_mode = CDAUDIO_PAUSED;
-      break;
-    case AS_PLAY_COMPLETED:
-      status->status_mode = CDAUDIO_COMPLETED;
-      break;
-    default:
-      status->status_mode = CDAUDIO_NOSTATUS;
-      break;
-   }
-#endif
    
    return 0;
 }
@@ -418,63 +368,27 @@ cd_play_frames(cd_desc, startframe, endframe)
    int endframe;
 #endif
 {
-#ifdef CDIOCPLAYMSF
-   struct ioc_play_msf cdmsf;
-#endif
-#ifdef CDROMPLAYMSF
-   struct cdrom_msf cdmsf;
-#endif
-#ifdef CDROM_PLAY_AUDIO_MSF
-   struct cd_play_audio_msf msf;
+   struct CDAUDIO_MSF cdmsf;
+
+#ifdef BROKEN_SOLARIS_LEADOUT
+   endframe -= 1;
 #endif
 
-#ifdef CDIOCPLAYMSF
-   cdmsf.start_m = startframe / 4500;
-   cdmsf.start_s = (startframe % 4500) / 75;
-   cdmsf.start_f = startframe % 75;
-   cdmsf.end_m = endframe / 4500;
-   cdmsf.end_s = (endframe % 4500) / 75;
-   cdmsf.end_f = endframe % 75;
-#endif
-#ifdef CDROMPLAYMSF
-   cdmsf.cdmsf_min0 = startframe / 4500;
-   cdmsf.cdmsf_sec0 = (startframe % 4500) / 75;
-   cdmsf.cdmsf_frame0 = startframe % 75;
-   cdmsf.cdmsf_min1 = endframe / 4500;
-   cdmsf.cdmsf_sec1 = (endframe % 4500) / 75;
-   cdmsf.cdmsf_frame1 = endframe % 75;
-#endif
-#ifdef CDROM_PLAY_AUDIO_MSF
-   msf.msf_starting_M_unit = startframe / 4500;
-   msf.msf_starting_S_unit = (startframe % 4500) / 75;
-   msf.msf_starting_F_unit = startframe % 75;
-   msf.msf_ending_M_unit = endframe / 4500;
-   msf.msf_ending_S_unit = (endframe % 4500) / 75;
-   msf.msf_ending_F_unit = endframe % 75;
-#endif
-   
-#ifdef CDIOCSTART
-   if(ioctl(cd_desc, CDIOCSTART) < 0)
-     return -1;
-#endif
-#ifdef CDROMSTART
-   if(ioctl(cd_desc, CDROMSTART) < 0)
-     return -1;
-#endif
-   
-#ifdef CDIOCPLAYMSF
-   if(ioctl(cd_desc, CDIOCPLAYMSF, (char *)&cdmsf) < 0)
-     return -1;
-#endif
-#ifdef CDROMPLAYMSF
-   if(ioctl(cd_desc, CDROMPLAYMSF, &cdmsf) < 0)
-     return -1;
-#endif
-#ifdef CDROM_PLAY_AUDIO_MSF
-   if(ioctl(cd_desc, CDROM_PLAY_AUDIO_MSF, &msf) < 0)
-     return -1;
+   cdmsf.CDMSF_START_M = startframe / 4500;
+   cdmsf.CDMSF_START_S = (startframe % 4500) / 75;
+   cdmsf.CDMSF_START_F = startframe % 75;
+   cdmsf.CDMSF_END_M = endframe / 4500;
+   cdmsf.CDMSF_END_S = (endframe % 4500) / 75;
+   cdmsf.CDMSF_END_F = endframe % 75;
+
+#ifdef CDAUDIO_START
+   /* Some CDROM drives don't support this so lets try things without it */
+   /* ioctl(cd_desc, CDAUDIO_START); */
 #endif
    
+   if(ioctl(cd_desc, CDAUDIO_PLAY_MSF, &cdmsf) < 0)
+     return -2;
+
    return 0;
 }
 
@@ -555,14 +469,8 @@ cd_stop(cd_desc)
    int cd_desc;
 #endif
 {
-#ifdef CDIOCSTOP
-   if(ioctl(cd_desc, CDIOCSTOP) < 0)
+   if(ioctl(cd_desc, CDAUDIO_STOP) < 0)
      return -1;
-#endif
-#ifdef CDROMSTOP
-   if(ioctl(cd_desc, CDROMSTOP) < 0)
-     return -1;
-#endif
    
    return 0;
 }
@@ -576,18 +484,8 @@ cd_pause(cd_desc)
    int cd_desc;
 #endif
 {
-#ifdef CDIOCPAUSE
-   if(ioctl(cd_desc, CDIOCPAUSE) < 0)
+   if(ioctl(cd_desc, CDAUDIO_PAUSE) < 0)
      return -1;
-#endif
-#ifdef CDROMPAUSE
-   if(ioctl(cd_desc, CDROMPAUSE) < 0)
-     return -1;
-#endif
-#ifdef CDROM_PAUSE_PLAY
-   if(ioctl(cd_desc, CDROM_PAUSE_PLAY) < 0)
-     return -1;
-#endif
    
    return 0;
 }
@@ -601,18 +499,8 @@ cd_resume(cd_desc)
    int cd_desc;
 #endif
 {
-#ifdef CDIOCRESUME
-   if(ioctl(cd_desc, CDIOCRESUME) < 0)
+   if(ioctl(cd_desc, CDAUDIO_RESUME) < 0)
      return -1;
-#endif
-#ifdef CDROMRESUME
-   if(ioctl(cd_desc, CDROMRESUME) < 0)
-     return -1;
-#endif
-#ifdef CDROM_RESUME_PLAY
-   if(ioctl(cd_desc, CDROM_RESUME_PLAY) < 0)
-     return -1;
-#endif
    
    return 0;
 }
@@ -626,14 +514,8 @@ cd_eject(cd_desc)
    int cd_desc;
 #endif
 {  
-#ifdef CDIOCEJECT
-   if(ioctl(cd_desc, CDIOCEJECT) < 0)
+   if(ioctl(cd_desc, CDAUDIO_EJECT) < 0)
      return -1;
-#endif
-#ifdef CDROMEJECT
-   if(ioctl(cd_desc, CDROMEJECT) < 0)
-     return -1;
-#endif
    
    return 0;
 }
@@ -647,16 +529,15 @@ cd_close(cd_desc)
    int cd_desc;
 #endif
 {
-#ifdef CDIOCCLOSE
-   if(ioctl(cd_desc, CDIOCCLOSE) < 0)
+#ifdef CDAUDIO_CLOSE
+   if(ioctl(cd_desc, CDAUDIO_CLOSE) < 0)
      return -1;
-#endif
-#ifdef CDROMCLOSETRAY
-   if(ioctl(cd_desc, CDROMCLOSETRAY) < 0)
-     return -1;
-#endif
    
    return 0;
+#else
+   errno = ENOTTY;
+   return -1;
+#endif
 }
 
 /* Return the current volume setting */
@@ -669,49 +550,34 @@ cd_get_volume(cd_desc, vol)
    struct disc_volume *vol;
 #endif
 {
-#ifdef CDIOCGETVOL
-   struct ioc_vol volume;
+#ifdef CDAUDIO_GET_VOLUME
+   struct CDAUDIO_VOLSTAT cdvol;
+#ifdef CDAUDIO_VOLSTAT_DATA
+   struct CDAUDIO_VOLSTAT_DATA cdvol_data;
+   cdvol.CDVOLSTAT_DATA = &cdvol_data;
+   cdvol.CDVOLSTAT_DATA_LEN = sizeof(cdvol_data);
 #endif
-#ifdef CDROMVOLREAD
-   struct cdrom_volctrl volume;
-#endif
-#ifdef CDROM_PLAYBACK_STATUS
-   struct cd_playback pb;
-   struct cd_playback_status ps;
-#endif
-   
-#ifdef CDIOCGETVOL
-   if(ioctl(cd_desc, CDIOCGETVOL, &volume) < 0)
+
+   if(ioctl(cd_desc, CDAUDIO_GET_VOLUME, &cdvol) < 0)
      return -1;
    
-   vol->vol_front.left = volume.vol[0];
-   vol->vol_front.right = volume.vol[1];
-   vol->vol_back.left = volume.vol[2];
-   vol->vol_back.right = volume.vol[3];
-#endif
-#ifdef CDROMVOLREAD
-   if(ioctl(cd_desc, CDROMVOLREAD, &volume) < 0)
-     return -1;
-      
-   vol->vol_front.left = volume.channel0;
-   vol->vol_front.right = volume.channel1;
-   vol->vol_back.left = volume.channel2;
-   vol->vol_back.right = volume.channel3;
-#endif
-#ifdef CDROM_PLAYBACK_STATUS
-   pb.pb_buffer = &ps;
-   pb.pb_alloc_length = sizeof(struct cd_playback_status);
-   
-   if(ioctl(cd_desc, CDROM_PLAYBACK_STATUS, &pb) < 0)
-     return -1;
-   
-   vol->vol_front.left = ps.ps_chan0_volume;
-   vol->vol_front.right = ps.ps_chan1_volume;
-   vol->vol_back.left = ps.ps_chan2_volume;
-   vol->vol_back.right = ps.ps_chan3_volume;
+   vol->vol_front.left = cdvol.CDVOLSTAT_FRONT_LEFT;
+   vol->vol_front.right = cdvol.CDVOLSTAT_FRONT_RIGHT;
+   vol->vol_back.left = cdvol.CDVOLSTAT_BACK_LEFT;
+   vol->vol_back.right = cdvol.CDVOLSTAT_BACK_RIGHT;
+
+#ifdef CDAUDIO_VOLSTAT_DATA
+   vol->vol_front.left = cdvol_data.CDVOLSTAT_FRONT_LEFT;
+   vol->vol_front.right = cdvol_data.CDVOLSTAT_FRONT_RIGHT;
+   vol->vol_back.left = cdvol_data.CDVOLSTAT_BACK_LEFT;
+   vol->vol_back.right = cdvol_data.CDVOLSTAT_BACK_RIGHT;
 #endif
    
    return 0;
+#else
+   errno = ENOTTY;
+   return -1;
+#endif
 }
 
 /* Set the volume */
@@ -724,55 +590,31 @@ cd_set_volume(cd_desc, vol)
    struct disc_volume vol;
 #endif
 {
-#ifdef CDIOCSETVOL
-   struct ioc_vol volume;
-#endif
-#ifdef CDROMVOLCTRL
-   struct cdrom_volctrl volume;
-#endif
-#ifdef CDROM_PLAYBACK_CONTROL
-   struct cd_playback pb;
-   struct cd_playback_control pc;
+   struct CDAUDIO_VOLCTRL cdvol;
+#ifdef CDAUDIO_VOLCTRL_DATA
+   struct cd_playback cdvol_data;
+   cdvol.CDVOLCTRL_DATA = &cdvol_data;
+   cdvol.CDVOLCTRL_DATA_LEN = sizeof(cdvol_data);
 #endif
    
    if(vol.vol_front.left > 255 || vol.vol_front.left < 0 || vol.vol_front.right > 255 || vol.vol_front.right < 0 || vol.vol_back.left > 255 || vol.vol_back.left < 0 || vol.vol_back.right > 255 || vol.vol_back.right < 0)
      return -1;
 
-#ifdef CDIOCSETVOL
-   volume.vol[0] = vol.vol_front.left;
-   volume.vol[1] = vol.vol_front.right;
-   volume.vol[2] = vol.vol_back.left;
-   volume.vol[3] = vol.vol_back.right;
-   
-   if(ioctl(cd_desc, CDIOCSETVOL, &volume) < 0)
-     return -1;
-#endif
-#ifdef CDROMVOLCTRL
-   volume.channel0 = vol.vol_front.left;
-   volume.channel1 = vol.vol_front.right;
-   volume.channel2 = vol.vol_back.left;
-   volume.channel3 = vol.vol_back.right;
-   
-   if(ioctl(cd_desc, CDROMVOLCTRL, &volume) < 0)
-     return -1;
-#endif
-#ifdef CDROM_PLAYBACK_CONTROL
-   pc.pc_chan0_select = CDROM_MAX_VOLUME;
-   pc.pc_chan0_volume = vol.vol_front.left;
-   pc.pc_chan1_select = CDROM_MAX_VOLUME;
-   pc.pc_chan1_volume = vol.vol_front.right;
-   pc.pc_chan2_select = CDROM_MAX_VOLUME;
-   pc.pc_chan2_volume = vol.vol_back.left;
-   pc.pc_chan3_select = CDROM_MAX_VOLUME;
-   pc.pc_chan3_volume = vol.vol_back.right;
-   
-   pb.pb_buffer = &pc;
-   pb.pb_alloc_length = sizeof(struct cd_playback_control);
-   
-   if(ioctl(cd_desc, CDROM_PLAYBACK_CONTROL, &pb) < 0)
-     return -1;
+   cdvol.CDVOLCTRL_FRONT_LEFT = vol.vol_front.left;
+   cdvol.CDVOLCTRL_FRONT_RIGHT = vol.vol_front.right;
+   cdvol.CDVOLCTRL_BACK_LEFT = vol.vol_back.left;
+   cdvol.CDVOLCTRL_BACK_RIGHT = vol.vol_back.right;
+
+#ifdef CDAUDIO_VOLCTRL_SELECT
+   cdvol_data.CDVOLCTRL_FRONT_LEFT_SELECT = CDAUDIO_MAX_VOLUME;
+   cdvol_data.CDVOLCTRL_FRONT_RIGHT_SELECT = CDAUDIO_MAX_VOLUME;
+   cdvol_data.CDVOLCTRL_BACK_LEFT_SELECT = CDAUDIO_MAX_VOLUME;
+   cdvol_data.CDVOLCTRL_BACK_RIGHT_SELECT = CDAUDIO_MAX_VOLUME;
 #endif
    
+   if(ioctl(cd_desc, CDAUDIO_SET_VOLUME, &cdvol) < 0)
+     return -1;
+
    return 0;
 }
 
@@ -943,21 +785,17 @@ cd_update(disc, status)
    struct disc_status status;
 #endif
 {
-   int pos;
-   
    if(!(disc->disc_present = status.status_present))
      return -1;
    
    disc->disc_mode = status.status_mode;
    memcpy(&disc->disc_time, &status.status_disc_time, sizeof(struct disc_timeval));
-
+   memcpy(&disc->disc_track_time, &status.status_track_time, sizeof(struct disc_timeval));
+   
    disc->disc_current_track = 0;
-   while(disc->disc_current_track < disc->disc_total_tracks && (disc->disc_time.minutes * 60 + disc->disc_time.seconds) * 75 + disc->disc_time.frames >= (disc->disc_track[disc->disc_current_track].track_pos.minutes * 60 + disc->disc_track[disc->disc_current_track].track_pos.seconds) * 75 + disc->disc_track[disc->disc_current_track].track_pos.frames)
+   while(disc->disc_current_track < disc->disc_total_tracks && 
+          cd_msf_to_frames(disc->disc_time) >= cd_msf_to_frames(disc->disc_track[disc->disc_current_track].track_pos) )
      disc->disc_current_track++;
-
-   pos = ((disc->disc_time.minutes * 60 + disc->disc_time.seconds) * 75 + disc->disc_time.frames) - ((disc->disc_track[disc->disc_current_track - 1].track_pos.minutes * 60 + disc->disc_track[disc->disc_current_track - 1].track_pos.seconds) * 75 + disc->disc_track[disc->disc_current_track - 1].track_pos.frames);
-
-   cd_frames_to_msf(&disc->disc_track_time, pos);
    
    return 0;
 }
@@ -983,12 +821,18 @@ cd_playctl(cd_desc, options, ...)
 
    if(options & PLAY_END_TRACK)
      end_track = va_arg(arglist, int);
-
+   else
+     end_track = disc.disc_total_tracks;
+   
    if(options & PLAY_START_POSITION)
      startpos = va_arg(arglist, struct disc_timeval *);
-
+   else
+     startpos = 0;
+   
    if(options & PLAY_END_POSITION)
      endpos = va_arg(arglist, struct disc_timeval *);
+   else
+     endpos = 0;
    
    va_end(arglist);
    
@@ -1018,5 +862,5 @@ cd_playctl(cd_desc, options, ...)
       end_position.frames = disc.disc_track[disc.disc_total_tracks].track_pos.frames;
    }
    
-   return cd_play_frames(cd_desc, (start_position.minutes * 60 + start_position.seconds) * 75 + start_position.frames, (end_position.minutes * 60 + end_position.seconds) * 75 + end_position.frames);
+   return cd_play_frames(cd_desc, cd_msf_to_frames(start_position), cd_msf_to_frames(end_position));
 }
