@@ -21,6 +21,15 @@ Boston, MA  02111-1307, USA.
 #ifndef _CDAUDIO_H
 #define _CDAUDIO_H
 
+#define LIBCDAUDIO_VERSION_MAJOR 0
+#define LIBCDAUDIO_VERSION_MINOR 99
+#define LIBCDAUDIO_VERSION_MICRO 4
+
+#define LIBCDAUDIO_VERSION \
+        ((LIBCDAUDIO_VERSION_MAJOR<<16)| \
+         (LIBCDAUDIO_VERSION_MINOR<< 8)| \
+         (LIBCDAUDIO_VERSION_MICRO))
+
 /* Used with disc_info */
 #define CDAUDIO_PLAYING				0
 #define CDAUDIO_PAUSED				1
@@ -86,6 +95,10 @@ Boston, MA  02111-1307, USA.
 #define PLAY_END_POSITION			4
 
 #ifndef __LIBCDAUDIO_INTERNAL
+#define cdindex_write_disc_data(cd_desc, data)	cdindex_write_data(cd_desc, &data)
+#define cddb_write_disc_data(cd_desc, data)	cddb_write_data(cd_desc, &data)
+#define cddb_erase_entry(data)			cddb_erase_data(&data)
+
 extern char cddb_message[256];
 extern int parse_disc_artist;
 extern int parse_track_artist;
@@ -93,6 +106,8 @@ extern int cddb_submit_method;
 extern char *cddb_submit_email_address;
 #endif
 
+extern long cdaudio_getversion(void);
+
 /* CDDB entry */
 struct cddb_entry {
    int entry_present;				/* Is there an entry? */
@@ -130,11 +145,18 @@ struct cddb_hello {
 /* An entry in the query list */
 struct query_list_entry {
    int list_genre;                              /* CDDB genre of entry */
-   int list_id;                                 /* CDDB ID of entry */
+   unsigned long list_id;                       /* CDDB ID of entry */
    char list_title[64];                         /* Title of entry */
    char list_artist[64];                        /* Entry's artist */
 };
 
+/* TOC data for CDDB query */
+struct cddb_direct_query_toc {
+   unsigned long toc_id;
+   int tracks;
+   int positions[MAX_TRACKS];
+};
+
 /* CDDB query structure */
 struct cddb_query {
 #define QUERY_NOMATCH				0
@@ -157,6 +179,8 @@ struct disc_status {
    int status_present;				/* Is disc present? */
    int status_mode;				/* Disc mode */
    struct disc_timeval status_disc_time;	/* Current disc time */
+   struct disc_timeval status_track_time; 	/* Current track time */
+   int status_current_track;			/* Current track */
 };
 
 /* Track specific information */
@@ -213,6 +237,14 @@ struct art_data {
    char art_data[DISC_ART_SIZE];		/* Stored image */
 };
 
+/* Compact cover art structure */
+struct art_mc_data {
+   int art_present;
+   int art_length;
+   char art_mime_type[16];
+   char *art_data;
+};
+
 /* Track database structure */
 struct track_data {
    char track_name[256];			/* Track name */
@@ -233,6 +265,33 @@ struct disc_data {
    struct track_data data_track[MAX_TRACKS];	/* Track names */
 };
 
+/* Compact track database structure */
+struct track_mc_data {
+   int track_name_len;
+   char *track_name;
+   int track_artist_len;
+   char *track_artist;
+   int track_extended_len;
+   char *track_extended;
+};
+
+/* Compact disc database structure */
+struct disc_mc_data {
+   unsigned long data_id;
+   char data_cdindex_id[CDINDEX_ID_SIZE];
+   int data_title_len;
+   char *data_title;
+   int data_artist_len;
+   char *data_artist;
+   int data_extended_len;
+   char *data_extended;
+   int data_genre;
+   int data_revision;
+   int data_artist_type;
+   int data_total_tracks;
+   struct track_mc_data **data_track;
+};
+
 /* Summary of a single disc in the changer */
 struct disc_summary {
    int disc_present;				/* Is disc present? */
@@ -250,6 +309,24 @@ struct disc_changer {
    struct disc_summary changer_disc[MAX_SLOTS];
 };
 
+/* Compact disc summary */
+struct disc_mc_summary {
+   int disc_present;
+   struct disc_timeval disc_length;
+   int disc_total_tracks;
+   unsigned long disc_id;
+   char data_cdindex_id[CDINDEX_ID_SIZE];
+   int disc_info_len;
+   char *disc_info;
+   struct art_mc_data data_art;
+};
+
+/* Compact disc changer structure */
+struct disc_mc_changer {
+   int changer_slots;
+   struct disc_mc_summary **changer_disc;
+};
+
 /* CDDB server list structure */
 struct cddb_serverlist {
    int list_len;
@@ -284,11 +361,16 @@ int cd_eject(int cd_desc);
 int cd_close(int cd_desc);
 int cd_set_volume(int cd_desc, struct disc_volume vol);
 int cd_get_volume(int cd_desc, struct disc_volume *vol);
-unsigned long cddb_discid(int cd_desc);
+long cddb_discid(int cd_desc);
 int cddb_process_url(struct cddb_host *host, const char *url);
 int cddb_read_serverlist(struct cddb_conf *conf, struct cddb_serverlist *list, struct cddb_server *proxy);
 int cddb_write_serverlist(struct cddb_conf conf, struct cddb_serverlist list, struct cddb_server proxy);
 char *cddb_genre(int genre);
+int cddb_direct_mc_alloc(struct disc_mc_data *data, int tracks);
+int cddb_mc_alloc(int cd_desc, struct disc_mc_data *data);
+void cddb_mc_free(struct disc_mc_data *data);
+int cddb_mc_copy_from_data(struct disc_mc_data *outdata, struct disc_data *indata);
+int cddb_data_copy_from_mc(struct disc_data *outdata, struct disc_mc_data *indata);
 int cddb_connect(struct cddb_server *server);
 int cddb_connect_server(struct cddb_host host, struct cddb_server *proxy, struct cddb_hello hello, ...);
 int cddb_skip_http_header(int sock);
@@ -300,8 +382,8 @@ int cddb_sites(int sock, int mode, struct cddb_serverlist *list, ...);
 int cddb_read_data(int desc, struct disc_data *data);
 int cddb_generate_unknown_entry(int cd_desc, struct disc_data *data);
 int cddb_read_disc_data(int cd_desc, struct disc_data *data);
-int cddb_write_disc_data(int cd_desc, struct disc_data data);
-int cddb_erase_entry(struct disc_data data);
+int cddb_write_data(int cd_desc, struct disc_data *data);
+int cddb_erase_data(struct disc_data *data);
 int cddb_stat_disc_data(int cd_desc, struct cddb_entry *entry);
 int cddb_http_query(int cd_desc, struct cddb_host host, struct cddb_hello hello, struct cddb_query *query);
 int cddb_http_proxy_query(int cd_desc, struct cddb_host host, struct cddb_server proxy, struct cddb_hello hello, struct cddb_query *query);
@@ -313,7 +395,7 @@ int cddb_http_submit(int cd_desc, struct cddb_host host, struct cddb_server *pro
 int cdindex_discid(int cd_desc, char *discid, int len);
 int cdindex_connect_server(struct cddb_host host, struct cddb_server *proxy, char *http_string, int len);
 int cdindex_read(int cd_desc, int sock, struct disc_data *data, char *http_string);
-int cdindex_write_disc_data(int cd_desc, struct disc_data data);
+int cdindex_write_data(int cd_desc, struct disc_data *data);
 int coverart_connect_server(struct cddb_host host, struct cddb_server *proxy, char *http_string, int len);
 int coverart_query(int cd_desc, int sock, struct art_query *query, char *http_string);
 int coverart_name_query(int sock, struct art_query *query, char *http_string, char *album, char *artist);
