@@ -1,6 +1,6 @@
 /*
 This is part of the audio CD player library
-Copyright (C)1998-99 Tony Arcieri
+Copyright (C)1998-99 Tony Arcieri <bascule@inferno.tusculum.edu>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -14,7 +14,8 @@ Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the
-Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+
+ Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 
 Based upon: 
@@ -24,6 +25,8 @@ from Peter C. Gutmann's implementation as found in
 Applied Cryptography by Bruce Schneier
 */
 
+/* XXX This file is NOT strtok() clean */
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -32,10 +35,15 @@ Applied Cryptography by Bruce Schneier
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <errno.h>
+
+#define __LIBCDAUDIO_INTERNAL
+
 #include <cdaudio.h>
 #include <cddb.h>
 #include <config.h>
-#include <errno.h>
+
+extern char *cddb_message;
 
 /* cdindex-tweaked base64 encoder */
 int cdindex_encode64(unsigned char *outbuffer, unsigned char *inbuffer, int inlen, int outlen);
@@ -285,9 +293,9 @@ sha_final(digest, sha)
 
 int
 #if __STDC__
-__internal_cdindex_discid(struct disc_info disc, char *discid, int len)
+cdindex_direct_discid(struct disc_info disc, char *discid, int len)
 #else
-__internal_cdindex_discid(disc, discid, len)
+cdindex_direct_discid(disc, discid, len)
    struct disc_info disc;
    char *discid;
    int len;
@@ -317,6 +325,19 @@ __internal_cdindex_discid(disc, discid, len)
    return cdindex_encode64(discid, digest, 20, len);
 }
 
+int
+#if __STDC__
+__internal_cdindex_discid(struct disc_info disc, char *discid, int len)
+#else
+__internal_cdindex_discid(disc, discid, len)
+   struct disc_info disc;
+   char *discid;
+   int len;
+#endif
+{
+   return cdindex_direct_discid(disc, discid, len);
+}
+
 int
 cdindex_discid(int cd_desc, char *discid, int len)
 {
@@ -529,15 +550,10 @@ cdindex_generate_new_entry(cd_desc, data)
   int cd_desc;
   struct disc_data *data;
 #endif
-{
-   struct disc_data outdata;
-   
+{  
    if(cddb_read_data(cd_desc, data) < 0)
      cddb_generate_unknown_entry(cd_desc, data);
    
-   memcpy(&outdata, data, sizeof(struct disc_data));
-   cddb_write_disc_data(cd_desc, outdata);
-   
    return 0;
 }
 
@@ -648,11 +664,11 @@ cdindex_read_disc_data(cd_desc, data)
 
 int
 #if __STDC__
-cdindex_write_disc_data(int cd_desc, struct disc_data data)
+cdindex_write_data(int cd_desc, struct disc_data *data)
 #else
-cdindex_write_disc_data(cd_desc, data)
+cdindex_write_data(cd_desc, data)
   int cd_desc;
-  struct disc_data data;
+  struct disc_data *data;
 #endif
 {
    FILE *cdindex_xml;
@@ -669,14 +685,14 @@ cdindex_write_disc_data(cd_desc, data)
    if(cd_stat(cd_desc, &disc) < 0)
      return -1;
   
-   if((data.data_id = __internal_cddb_discid(disc)) < 0)
+   if((data->data_id = __internal_cddb_discid(disc)) < 0)
      return -1;
    
-   if(__internal_cdindex_discid(disc, data.data_cdindex_id, CDINDEX_ID_SIZE) < 0)
+   if(__internal_cdindex_discid(disc, data->data_cdindex_id, CDINDEX_ID_SIZE) < 0)
      return -1;
    
    snprintf(root_dir, 256, "%s/.cdindex", getenv("HOME"));
-   snprintf(file, 256, "%s/%s", root_dir, data.data_cdindex_id);
+   snprintf(file, 256, "%s/%s", root_dir, data->data_cdindex_id);
    
    if(stat(root_dir, &st) < 0) {
       if(errno != ENOENT)
@@ -696,11 +712,11 @@ cdindex_write_disc_data(cd_desc, data)
    fputs("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n", cdindex_xml);
    fputs("<!DOCTYPE CDInfo SYSTEM \"http://www.freeamp.org/dtd/CDInfo.dtd\">\n\n", cdindex_xml);
    fputs("<CDInfo>\n\n", cdindex_xml);
-   fprintf(cdindex_xml, "   <Title>%s</Title>\n", data.data_title);
+   fprintf(cdindex_xml, "   <Title>%s</Title>\n", data->data_title);
    fprintf(cdindex_xml, "   <NumTracks>%d</NumTracks>\n\n", disc.disc_total_tracks);
    fputs("   <IdInfo>\n", cdindex_xml);
    fputs("      <DiskId>\n", cdindex_xml);
-   fprintf(cdindex_xml, "         <Id>%s</Id>\n", data.data_cdindex_id);
+   fprintf(cdindex_xml, "         <Id>%s</Id>\n", data->data_cdindex_id);
    fprintf(cdindex_xml, "         <TOC First=\"%d\" Last=\"%d\">\n", disc.disc_first_track, disc.disc_total_tracks);
    fprintf(cdindex_xml, "            <Offset Num=\"0\">%d</Offset>\n", disc.disc_track[disc.disc_total_tracks].track_lba);
    for(tracks = 0; tracks < disc.disc_total_tracks; tracks++)
@@ -708,12 +724,12 @@ cdindex_write_disc_data(cd_desc, data)
    fputs("         </TOC>\n", cdindex_xml);
    fputs("      </DiskId>\n", cdindex_xml);
    fputs("   </IdInfo>\n\n", cdindex_xml);
-   if(strcmp(data.data_artist, "(various)") != 0) {
+   if(strcmp(data->data_artist, "(various)") != 0) {
       fputs("   <SingleArtistCD>\n", cdindex_xml);
-      fprintf(cdindex_xml, "      <Artist>%s</Artist>\n", data.data_artist);
+      fprintf(cdindex_xml, "      <Artist>%s</Artist>\n", data->data_artist);
       for(tracks = 0; tracks < disc.disc_total_tracks; tracks++) {
 	 fprintf(cdindex_xml, "      <Track Num=\"%d\">\n", tracks + 1);
-	 fprintf(cdindex_xml, "         <Name>%s</Name>\n", data.data_track[tracks].track_name);
+	 fprintf(cdindex_xml, "         <Name>%s</Name>\n", data->data_track[tracks].track_name);
 	 fputs("      </Track>\n", cdindex_xml);
       }
       fputs("   </SingleArtistCD>\n\n", cdindex_xml);
@@ -721,8 +737,8 @@ cdindex_write_disc_data(cd_desc, data)
       fputs("   <MultipleArtistCD>\n", cdindex_xml);
       for(tracks = 0; tracks < disc.disc_total_tracks; tracks++) {
 	 fprintf(cdindex_xml, "      <Track Num=\"%d\">\n", tracks + 1);
-	 fprintf(cdindex_xml, "         <Artist>%s</Artist>\n", data.data_track[tracks].track_artist);
-	 fprintf(cdindex_xml, "         <Name>%s</Name>\n", data.data_track[tracks].track_name);
+	 fprintf(cdindex_xml, "         <Artist>%s</Artist>\n", data->data_track[tracks].track_artist);
+	 fprintf(cdindex_xml, "         <Name>%s</Name>\n", data->data_track[tracks].track_name);
 	 fputs("      </Track>\n", cdindex_xml);
       }
       fputs("   </MultipleArtistCD>\n\n", cdindex_xml);
@@ -733,6 +749,116 @@ cdindex_write_disc_data(cd_desc, data)
    return 0;
 }
 
+int
+#if __STDC__
+cdindex_mc_read(int cd_desc, int sock, struct disc_mc_data *data, char *http_string)
+#else
+cdindex_mc_read(cd_desc, sock, data, http_string)
+   int cd_desc;
+   int sock;
+   struct disc_mc_data *data;
+   char *http_string;
+#endif
+{
+   struct disc_data *indata;
+   
+   if((indata = malloc(sizeof(struct disc_data))) == NULL)
+     return -1;
+   
+   if(cdindex_read(cd_desc, sock, indata, http_string) < 0) {
+      free(indata);
+      return -1;
+   }
+   
+   if(cddb_mc_alloc(cd_desc, data) < 0) {
+      free(indata);
+      return -1;
+   }
+   
+   if(cddb_mc_copy_from_data(data, indata) < 0) {
+      free(indata);
+      cddb_mc_free(data);
+      return -1;
+   }
+   
+   free(indata);
+   return 0;
+}
+
+int
+#if __STDC__
+cdindex_mc_read_disc_data(int cd_desc, struct disc_mc_data *data)
+#else
+cdindex_mc_read_disc_data(cd_desc, data)
+   int cd_desc;
+   struct disc_mc_data *data;
+#endif
+{
+   struct disc_data *indata;
+   
+   if((indata = malloc(sizeof(struct disc_data))) == NULL)
+     return -1;
+   
+   if(cdindex_read_disc_data(cd_desc, indata) < 0) {
+      free(indata);
+      return -1;
+   }
+   
+   if(cddb_mc_alloc(cd_desc, data) < 0) {
+      free(indata);
+      return -1;
+   }
+   
+   if(cddb_mc_copy_from_data(data, indata) < 0) {
+      free(indata);
+      cddb_mc_free(data);
+      return -1;
+   }
+   
+   free(indata);
+   return 0;
+}
+
+int
+#if __STDC__
+cdindex_mc_write_disc_data(int cd_desc, struct disc_mc_data *data)
+#else
+cdindex_mc_write_disc_data(cd_desc, data)
+   int cd_desc;
+   struct disc_mc_data *data;
+#endif
+{
+   struct disc_data *outdata;
+   
+   if((outdata = malloc(sizeof(struct disc_data))) == NULL)
+     return -1;
+   
+   if(cddb_data_copy_from_mc(outdata, data) < 0) {
+      free(outdata);
+      return -1;
+   }
+   
+   if(cdindex_write_data(cd_desc, outdata) < 0) {
+      free(outdata);
+      return -1;
+   }
+   
+   free(outdata);
+   return 0;
+}
+
+int
+#if __STDC__
+cdindex_mc_generate_new_entry(int cd_desc, struct disc_mc_data *data)
+#else
+cdindex_mc_generate_new_entry(cd_desc, data)
+   int cd_desc;
+   struct disc_mc_data *data;
+#endif
+{
+   return cddb_mc_generate_new_entry(cd_desc, data);
+}
+
 int
 #if __STDC__
 cdindex_stat_disc_data(int cd_desc, struct cddb_entry *entry)
